@@ -1,5 +1,5 @@
 package Git::Raw::Packbuilder;
-$Git::Raw::Packbuilder::VERSION = '0.54'; # TRIAL
+$Git::Raw::Packbuilder::VERSION = '0.55'; # TRIAL
 use strict;
 use warnings;
 use Carp;
@@ -25,11 +25,11 @@ use Git::Raw;
 
 =head1 NAME
 
-Git::Raw::Packbuilder - Packbuilder class
+Git::Raw::Packbuilder - Git packbuilder class
 
 =head1 VERSION
 
-version 0.54
+version 0.55
 
 =head1 AUTHOR
 
